----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Interfaces; use Interfaces;
with Oled.Draw;

package body Oled.Buffered.Float_Oper is

   function Min (X1, X2 : Unsigned_8) return Unsigned_8
   renames Unsigned_8'Min;

   function Max (X1, X2 : Unsigned_8) return Unsigned_8
   renames Unsigned_8'Max;

   ---------------
   -- Draw_Line --
   ---------------

   procedure Draw_Line (Bmp             : in out Bitmap;
                        X1, Y1, X2, Y2  : in     Unsigned_8;
                        Oper            : in     Bit_Oper := Bit_Copy)
   is
      A, C : Float;
   begin
      -- vertical or horizontal line - need to use optimized proc
      if Y1 = Y2 then
         Draw_Horiz_Line (Bmp, Y => Y1, X1 => X1, X2 => X2, Oper => Oper);
         return;
      end if;

      if X1 = X2 then
         Draw_Vertic_Line (Bmp, X => X1, Y1 => Y1, Y2 => Y2, Oper => Oper);
         return;
      end if;

      --  Y = A*X + C
      A := (Float (Y2) - Float (Y1))
        / (Float (X2) - Float (X1));
      C := ((Float (Y2) + Float (Y1))
              - A * (Float (X2) + Float (X1))) / 2.0 ;

      if abs A < 1.0 then
         for X in Min (X1, X2) .. Max (X1, X2) loop
            Set_Pixel (Bmp,
                       X => X,
                       Y => Unsigned_8
                         (Float'Machine_Rounding
                            (A * Float (X) + C)),
                      Oper => Oper);
         end loop;

      else
         for Y in Min (Y1, Y2) .. Max (Y1, Y2) loop
            Set_Pixel (Bmp,
                       X => Unsigned_8
                         (Float'Machine_Rounding
                            ((Float (Y) - C) / A)),
                       Y => Y,
                       Oper => Oper);
         end loop;
      end if;
   end Draw_Line;

end Oled.Buffered.Float_Oper;
