----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Oled.Draw; use Oled.Draw;

package body Oled.Chart is
   Chart_First_Page : Oled_Page_Num := 0;
   Chart_Last_Page  : Oled_Page_Num := 3;

   Cur_Chart_Col   : Oled.Oled_Col_Num := 0;

   Pix_Height : Unsigned_8 := 0;
   Col_Width  : Unsigned_8 := 1;
   Col_Space  : Unsigned_8 := 0;

   ------------------
   -- Clear_Chart  --
   ------------------

   procedure Clear_Chart is
   begin
      for J in Chart_First_Page .. Chart_Last_Page loop
         Oled.Clear_Page (J);
      end loop;

      Cur_Chart_Col := 0;
      Set_Cursor (Chart_First_Page, 0);
   end Clear_Chart;

   ----------------
   -- Draw_Chart --
   ----------------

   procedure Draw_Chart (Heigh, Low : Oled_Line_Num)
   is

      --   Chart_Last_Page - 0-axe of chart
      Hi_Page  : constant Unsigned_8 := Chart_Last_Page - Heigh / 8;
      Hi_Bit   : constant Unsigned_8 := Heigh mod 8;
      Lo_Page  : constant Unsigned_8 := Chart_Last_Page - Low / 8;
      Lo_Bit   : constant Unsigned_8 := Low mod 8;
      B, B1    : Unsigned_8 := 0;
   begin
      if Heigh < Low
        or else Heigh  >= Pix_Height
        or else Low >= Pix_Height
      then
         raise Program_Error;
      end if;

      for P in Chart_First_Page  .. Chart_Last_Page loop
         if P = Hi_Page then
            B := Shift_Left (16#FF#, 7 - Integer (Hi_Bit));

         end if;

         if P = Lo_Page then
            B1 := Shift_Right (16#FF#, Integer (Lo_Bit));

            if P = Hi_Page then
               B := B and B1;
            else
               B := B1;
            end if;
         end if;

         if P > Hi_Page and P < Lo_Page then
            B := 16#FF#;
         elsif P < Hi_Page or P > Lo_Page then
            B := 16#00#;
         end if;

         Set_Cursor (P, Cur_Chart_Col);

         for K in 1 .. Col_Width loop
            Set_Column_Data (B);
         end loop;
      end loop;

      Cur_Chart_Col := Cur_Chart_Col + Col_Width;

      if Col_Space > 0 then
         B := 16#00#;

         for P in Chart_First_Page  .. Chart_Last_Page loop
            Set_Cursor (P, Cur_Chart_Col);

            for K in 1 .. Col_Space loop
               Set_Column_Data (B);
            end loop;
         end loop;
      end if;

      Cur_Chart_Col := Cur_Chart_Col + Col_Space;
   end Draw_Chart;

   ----------------
   -- Init_Chart --
   ----------------

   procedure Init_Chart (First_Row    : Oled_Page_Num := 0;
                         Last_Row     : Oled_Page_Num := 3;
                         Column_Width : Unsigned_8 := 1;
                         Column_Space : Unsigned_8 := 0) is
   begin
      Chart_First_Page := First_Row;
      Chart_Last_Page := Last_Row;
      Pix_Height := (Last_Row - First_Row + 1) * 8;

      if Pix_Height <= 0
        or else Column_Width > 32
        or else Column_Space > 32
      then
         raise Program_Error;
      end if;

      Col_Width := Column_Width;
      Col_Space := Column_Space;
      Cur_Chart_Col := 0;

      Set_Cursor (First_Row, 0);
   end Init_Chart;

end Oled.Chart;
