----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Font; use Font;

package body Oled.Draw is

   -----------------
   -- Draw_Bitmap --
   -----------------

   procedure Draw_Bitmap (Bmp            : Bitmap;
                          Bmp_Start_Row  : Unsigned_8;
                          Bmp_End_Row    : Unsigned_8;
                          Bmp_Start_Col  : Unsigned_8;
                          Bmp_End_Col    : Unsigned_8;
                          Oled_Start_Row : Oled_Page_Num;
                          Oled_Start_Col : Oled_Col_Num)
   is
   begin
      for R in Bmp_Start_Row .. Bmp_End_Row loop
         Set_Cursor (R - Bmp_Start_Row + Oled_Start_Row, Oled_Start_Col);

         Draw_Bitmap_One_Row (Bmp           => Bmp,
                              Bmp_Row       => R,
                              Bmp_Start_Col => Bmp_Start_Col,
                              Bmp_End_Col   => Bmp_End_Col);
      end loop;
   end Draw_Bitmap;

   -------------------------
   -- Draw_Bitmap_One_Row --
   -------------------------

   procedure Draw_Bitmap_One_Row (Bmp           : Bitmap;
                                  Bmp_Row       : Unsigned_8;
                                  Bmp_Start_Col : Unsigned_8;
                                  Bmp_End_Col   : Unsigned_8)
   is
   begin
      for C in Bmp_Start_Col .. Bmp_End_Col loop
         Set_Column_Data (Bmp (Bmp_Row, C));
      end loop;
   end Draw_Bitmap_One_Row;

   ----------------
   -- Draw_Glyph --
   ----------------

   procedure Draw_Glyph (Glyph  : Glyph_Type)
   is
   begin
      Draw_Bitmap_One_Row (Bmp           => Glyph,
                           Bmp_Row       => 0,
                           Bmp_Start_Col => 0,
                           Bmp_End_Col   => Glyph'Last (2));
   end Draw_Glyph;

   -----------------
   -- Draw_String --
   -----------------

   procedure Draw_String (Str  : String;
                          Row  : Oled_Page_Num;
                          X    : Oled_Col_Num)
   is
   begin
      Set_Cursor (Row, X);
      Draw_String (Str);
   end Draw_String;

   procedure Draw_String (Str  : String) is
   begin
      for I in Str'First .. Str'Last loop
         Draw_Glyph (Font.Get_Glyph (Str (I)));
      end loop;
   end Draw_String;

   ---------------------
   -- Set_Column_Data --
   ---------------------

   procedure Set_Column_Data (Row  : Oled_Page_Num;
                              X    : Oled_Col_Num;
                              Data : Unsigned_8)
   is
   begin
      Set_Cursor (Row, X);
      Oled.Set_Column_Data (Data);
   end Set_Column_Data;

   ----------------
   -- Set_Cursor --
   ----------------

   procedure Set_Cursor (Row  : Oled_Page_Num;
                         X    : Oled_Col_Num) is
   begin
      Oled.Set_Page (Row);
      Oled.Set_Column (X);
   end Set_Cursor;

end Oled.Draw;
