----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Oled.Draw; use Oled.Draw;
with Font;

package body Oled.Text is

   First_Print_Page : Oled_Page_Num := 0;
   Last_Print_Page  : Oled_Page_Num := 3;

   subtype Cursor_Row is Unsigned_8
     range Oled_Page_Num'First .. Oled_Page_Num'Last + 1;
   subtype Cursor_Column is Unsigned_8
     range Oled_Col_Num'First .. Oled_Col_Num'Last + 1;

   Cur_Print_Page  : Cursor_Row    :=  First_Print_Page;
   Cur_Print_Col   : Cursor_Column := 0;

   ---------
   -- Put --
   ---------

   procedure Put (Str : String) is
   begin
      if End_Of_Text_Buffer then
         return;
      end if;

      Set_Cursor (Row => Cur_Print_Page, X => Cur_Print_Col);

      for I in Str'First .. Str'Last loop

         if Cur_Print_Col + Font.Font_Width - 1 > Oled_Col_Num'Last then
            if End_Of_Text_Buffer then
               exit;
            else
               New_Line;

               if End_Of_Text_Buffer then
                  exit;
               end if;
            end if;
         end if;

         Draw_Glyph (Font.Get_Glyph (Str (I)));
         Cur_Print_Col := Cur_Print_Col + Font.Font_Width;
      end loop;
    end Put;

   --------------
   -- Put_Line --
   --------------

   procedure Put_Line (Str : String) is
   begin
      Put (Str);
      New_Line;
   end Put_Line;

   --------------
   -- New_Line --
   --------------

   procedure New_Line
   is
   begin
      if Cur_Print_Page <= Last_Print_Page then -- end of text buffer
         Cur_Print_Page := Cur_Print_Page + 1;
         Cur_Print_Col := 0;

         if Cur_Print_Page < Last_Print_Page then
            Set_Cursor (Row => Cur_Print_Page, X => 0);
         end if;
      end if;
   end New_Line;

   -------------------
   -- End_Of_Buffer --
   -------------------

   function End_Of_Text_Buffer return Boolean is
   begin
      return Cur_Print_Page > Last_Print_Page;
   end End_Of_Text_Buffer;

   ----------------------
   -- Set_Print_Buffer --
   ----------------------

   procedure Set_Text_Buffer (First_Row, Last_Row  : Oled_Page_Num) is
   begin
      First_Print_Page := First_Row;
      Last_Print_Page := Last_Row;
      Cur_Print_Page := First_Row;
      Cur_Print_Col := 0;

      Clear_Text_Buffer;
   end Set_Text_Buffer;

   ------------------------
   -- Clear_Text_Buffer --
   ------------------------

   procedure Clear_Text_Buffer is
   begin
      for J in First_Print_Page .. Last_Print_Page loop
         Oled.Clear_Page (J);
      end loop;
      Set_Cursor (Row => First_Print_Page, X => 0);
   end Clear_Text_Buffer;

end Oled.Text;
