----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Interfaces; use Interfaces;

package Oled is
   subtype Oled_Col_Num is Unsigned_8 range 0 .. 127;
   subtype Oled_Page_Num is Unsigned_8 range 0 .. 7;
   Oled_Page_Height : constant := 8;
   --  Number of pixels per byte
   Oled_Pixels_Per_Byte : constant := 8;

   subtype Oled_Line_Num is Unsigned_8
     range 0 .. Oled_Page_Height * (Oled_Page_Num'Last + 1) - 1;

   procedure Invert (Val : Boolean);

   --  Val in range 0 .. 63
   procedure Start_Line (Val : Oled_Line_Num);

   --  Clear the screen
   procedure Clear;

   procedure Clear_Page (Page : Oled_Page_Num);

   --  Initialize the Oled.  Must be done once
   procedure Init;

   --  Set the current page (and clear column)
   procedure Set_Page (Page : Oled_Page_Num);

   procedure Set_Column (Val : Oled_Col_Num);

   --  Draw 8 pixels at current position and increase position to the next column
   procedure Set_Column_Data (B : Unsigned_8);

end Oled;
