----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
-- This file based on gnat example, Copyright (C) 2013-2014, AdaCore      --
-- modified by Turishev Evgeniy 2015                                      --
----------------------------------------------------------------------------

with System;
with SAM4S; use SAM4S;
with Ada.Interrupts.Names;
with Ada.Real_Time;

package body Oled_Drv is

   -----------------
   -- Reset_Delay --
   -----------------

   procedure Reset_Delay
   is
      use Ada.Real_Time;
   begin
      delay until Clock + Microseconds (3);
   end Reset_Delay;

   --------------
   -- Spi_Prot --
   --------------

   protected Spi_Prot is
      pragma Interrupt_Priority (System.Interrupt_Priority'First);

      procedure Write_Byte (Cmd : Unsigned_8);

      procedure Interrupt;
      pragma Attach_Handler (Interrupt, Ada.Interrupts.Names.SPI_Interrupt);

      entry Wait_Tx;
   private
      Tx_Done : Boolean := False;
   end Spi_Prot;

   protected body Spi_Prot is
      procedure Write_Byte (Cmd : Unsigned_8) is
      begin
         --  Set PCS #2
         SPI.SPI_MR := (SPI.SPI_MR and not SPI_MR.PCS_Mask)
           or (SPI_MR.PCS_Mask and not ((2 ** 2) * SPI_MR.PCS));

         --  Write cmd
         SPI.SPI_TDR := Word (Cmd) * SPI_TDR.TD;

         --  Enable TXEMPTY interrupt.
         SPI.SPI_IER := SPI_SR.TXEMPTY;
      end Write_Byte;

      procedure Interrupt is
      begin
         if (SPI.SPI_SR and SPI_SR.TXEMPTY) /= 0 then
            --  Disable TXEMPTY interrupt.
            SPI.SPI_IDR := SPI_SR.TXEMPTY;

            Tx_Done := True;
         end if;
      end Interrupt;

      entry Wait_Tx when Tx_Done is
      begin
         Tx_Done := False;

         --  Deselect device
         SPI.SPI_MR := SPI.SPI_MR or SPI_MR.PCS_Mask;

         --  Last transfer
         SPI.SPI_CR := SPI_CR.LASTXFER;
      end Wait_Tx;
   end Spi_Prot;

   ----------------
   -- Wait_Ready --
   ----------------

   procedure Wait_Ready is
   begin
      null;
   end Wait_Ready;
   -----------
   -- Write --
   -----------

   procedure Write (Byte : Unsigned_8) is
   begin
      Spi_Prot.Write_Byte (Byte);
      --  need wait end of transmtion
      Spi_Prot.Wait_Tx;
   end Write;
end Oled_Drv;
