----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with SAM4S; use SAM4S;
with XSTK_Board;


package body Timer is
   Clock_Frequency : constant := 120_000_000;
   Tick_Period   : constant Unsigned_32 := Clock_Frequency / 1000;
   -- period = 1 ms

   Tick_Cnt : Tick_Count := 0;
   pragma Atomic (Tick_Cnt);


   Callback : Timer_Callback;

   procedure Timer_Intr_Handler;
   pragma Export (Asm, Timer_Intr_Handler, "SysTick_Handler");

   ----------
   -- Init --
   ----------

   procedure Init is
   begin
      SYST.CSR := SYST_CSR.Clk_Source; -- disable clock
      SYST.RVR := Word (Tick_Period - 1);
      SYST.CVR := 0;
      SYST.CSR := SYST_CSR.Clk_Source
        or SYST_CSR.Enable or SYST_CSR.Tick_Int;
   end Init;

   function Ticks return Tick_Count is
   begin
      return Tick_Cnt;
   end Ticks;

   procedure Timer_Intr_Handler is
   begin
      Tick_Cnt := Tick_Cnt + 1;

      if Callback /= null then
         Callback.all (Tick_Cnt);
      end if;
   end Timer_Intr_Handler;

   procedure Wait (Cnt : Tick_Count)
   is
      Next_Time : constant Tick_Count := Tick_Cnt + Cnt;
   begin
      while Tick_Cnt < Next_Time loop
         null;
      end loop;
   end Wait;

   procedure Set_Callback (Cb : Timer_Callback) is
   begin
      Callback := Cb;
   end Set_Callback;
end Timer;
