----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

pragma Restrictions (No_Elaboration_Code);
with SAM4S; use SAM4S;
with System; use System;

package Adc is
   ADC_ID : constant := 29; -- device_id

   ADC_Base : constant := 16#4003_8000#;

   type ADC_Data_Reg_Array is array (0 .. 15) of Word;

   type ADC_Registers is record
      CR    : Word;  --  Control Register
      MR    : Word;  --  Mode Register
      SEQR1 : Word;  --  Channel Sequence Register 1
      SEQR2 : Word;  --  Channel Sequence Register 2
      CHER  : Word;  --  Channel Enable Register
      CHDR  : Word;  --  Channel Disable Register
      CHSR  : Word;  --  Channel Status Register
      RSRV1 : Word;  --  Reserved
      LCDR  : Word;  --  Last Converted Data Register
      IER   : Word;  --  Interrupt Enable Register
      IDR   : Word;  --  Interrupt Disable Register
      IMR   : Word;  --  Interrupt Mask Register
      ISR   : Word;  --  Interrupt Status Register
      OVER  : Word;  --  Overrun Status Register
      RSRV2 : Word;  --  Reserved
      RSRV3 : Word;  --  Reserved
      EMR   : Word;  --  Extended Mode Register
      CWR   : Word;  --  Compare Window Register
      CGR   : Word;  --  Channel Gain Register
      COR   : Word;  --  Channel Offset Register
      CDR   : ADC_Data_Reg_Array;  --  Channel Data Registers
      RSRV4 : Word;  --  Reserved
      ACR   : Word;  --  Analog Control Register
   end record;

   type ADC_WP_Registers is record
      WPMR  : Word;  --  Write Protection Mode Register
      WPSR  : Word;  --  Write Protection Status Register
   end record;

   package ADC_CR is
      SWRST    : constant := 1;
      START    : constant := 2;
      AUTOCAL  : constant := 4;
   end ADC_CR;

   ADC : ADC_Registers with Volatile, Import,
                            Address => System'To_Address (ADC_Base);

   ADC_WP : ADC_WP_Registers  with Volatile, Import,
                            Address => System'To_Address (ADC_Base + 16#E4#);


   subtype ADC_Startup_Time_Type is Natural range 0 .. 15;

   procedure Init (Clock_HZ : Natural; Startup_Time : ADC_Startup_Time_Type);

   subtype ADC_Channel_Type is Natural range 0 .. 15;
   procedure Enable_Channel (Channel : ADC_Channel_Type);

   subtype ADC_Trigger_Type is Byte range 0 .. 15;

   package ADC_Trigger is
      Software : constant ADC_Trigger_Type := 0;
      External : constant ADC_Trigger_Type := 1;
      Timer_0  : constant ADC_Trigger_Type := 1 * 2 + 1;
      Timer_1  : constant ADC_Trigger_Type := 2 * 2 + 1;
      Timer_2  : constant ADC_Trigger_Type := 3 * 2 + 1;
      PWM_0    : constant ADC_Trigger_Type := 4 * 2  + 1;
      PWM_1    : constant ADC_Trigger_Type := 5 * 2 + 1;
   end ADC_Trigger;

   procedure Config_Trigger (Trig : ADC_Trigger_Type; Free_Run : Boolean);

   procedure Start;

   function Get_Value (Channel : ADC_Channel_Type) return Word;
end Adc;
