----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with System;
with SAM4S; use SAM4S;

package body IO_Ports is

  type  PIO_Access is access all PIO_Registers;

   Ports : constant array (Port_Id'Range)
     of PIO_Access := (PIOA'Unrestricted_Access,
                       PIOB'Unrestricted_Access,
                       PIOC'Unrestricted_Access);

   Port_Dev_Id : constant array (Port_Id'Range) of Device_Id
     := (PIOA_ID, PIOB_ID, PIOC_ID);

   -----------
   -- Check --
   -----------

   function Check (Port : Port_Id; Pin : Port_Pin) return Boolean is
   begin
      return (Ports (Port).PDSR and 2 ** Pin) = 0;
   end Check;

   ---------------------
   -- Check_Interrupt --
   ---------------------

   function Check_Interrupt (Port : Port_Id; Pin : Port_Pin)
                            return Boolean is
   begin
      return (Ports (Port).ISR and 2 ** Pin) /= 0;
   end Check_Interrupt;

   -----------
   -- Clear --
   -----------

   procedure Clear (Port : Port_Id; Pin : Port_Pin) is
   begin
      Ports (Port).CODR := 2 ** Pin;
   end Clear;

   -----------------------
   -- Disable_Interrupt --
   -----------------------

   procedure Disable_Interrupt (Port : Port_Id; Pin : Port_Pin)
   is
   begin
      Ports (Port).IDR := 2 ** Pin;
   end Disable_Interrupt;

   procedure Port_Config (Port : Port_Id; Mask : Word; Mode : Port_Mode)
   is
   begin
      Enable_Clock (Port_Dev_Id (Port));
      Ports (Port).PER := Mask;

      case Mode is
         when Port_Mode_In =>
            Ports (Port).PUER := Mask;
         when Port_Mode_Out =>
            Ports (Port).OER := Mask;
            Ports (Port).CODR := Mask;
            Ports (Port).MDDR := Mask;
         when others => null;
      end case;
   end Port_Config;

   procedure Port_Config (Port : Port_Id; Pin : Port_Pin; Mode : Port_Mode)
   is
      Mask : constant Word := 2 ** Pin;
   begin
      Port_Config (Port => Port, Mask => Mask, Mode => Mode);
   end Port_Config;

   ------------------
   -- Enable_Clock --
   ------------------

   procedure Enable_Clock (Dev_Id : Device_Id)
   is
   begin
      PMC.PMC_PCER0 := 2 ** Dev_Id;
   end Enable_Clock;

   ----------------------
   -- Enable_Interrupt --
   ----------------------

   procedure Enable_Interrupt (Port : Port_Id; Pin : Port_Pin)
   is
   begin
      Ports (Port).IER := 2 ** Pin;
   end Enable_Interrupt;

   ---------
   -- Set --
   ---------

   procedure Set (Port : Port_Id; Pin : Port_Pin) is
   begin
      Ports (Port).SODR := 2 ** Pin;
   end Set;

   ------------
   -- Toogle --
   ------------

   procedure Toogle (Port : Port_Id; Pin : Port_Pin)
   is
      Mask : Word := 2 ** Pin;
   begin
      if (Ports (Port).ODSR and Mask) /= 0 then
         Ports (Port).CODR := Mask;
      else
         Ports (Port).SODR := Mask;
      end if;
   end Toogle;


end IO_Ports;
