----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with SAM4S; use SAM4S;
with System;
with IO_Ports; use IO_Ports;

package body XSTK_Oled1_Board is

   type PIO_Addr is record
      Port : IO_Ports.Port_Id;
      Pin  : IO_Ports.Port_Pin;
   end record;

   Leds : constant array (Led_Num'Range) of PIO_Addr :=
     ((Port => Port_C, Pin =>  20),
      (Port => Port_A, Pin =>  16),
      (Port => Port_C, Pin =>  22));

   Butts : constant array (Button_Num'Range) of PIO_Addr :=
     ((Port => Port_A, Pin =>  0),
      (Port => Port_C, Pin =>  29),
      (Port => Port_C, Pin =>  30));

   ----------
   -- Init --
   ----------

   procedure Init is
   begin
      for J in Led_Num'Range loop
         Port_Config (Port => Leds (J).Port,
                      Pin => Leds (J).Pin,
                      Mode => Port_Mode_Out);
      end loop;

      for J in Button_Num'Range loop
         Port_Config (Port => Butts (J).Port,
                      Pin => Butts (J).Pin,
                      Mode => Port_Mode_In);
      end loop;
   end Init;

   function Button (Num : Button_Num) return Boolean is
   begin
      return IO_Ports.Check (Butts (Num).Port, Butts (Num).Pin);
   end Button;

   function Button_Port (Num : Button_Num) return IO_Ports.Port_Id is
   begin
      return Butts (Num).Port;
   end Button_Port;

   function Button_Pin (Num : Button_Num) return IO_Ports.Port_Pin is
   begin
      return Butts (Num).Pin;
   end Button_Pin;

   procedure Led_On (Num : Led_Num) is
   begin
      IO_Ports.Clear (Leds (Num).Port, Leds (Num).Pin);
   end Led_On;

   procedure Led_Off (Num : Led_Num) is
   begin
      IO_Ports.Set (Leds (Num).Port, Leds (Num).Pin);
   end Led_Off;

   procedure Led_Toggle (Num : Led_Num) is
   begin
      IO_Ports.Toogle (Leds (Num).Port, Leds (Num).Pin);
   end Led_Toggle;

   procedure Enable_Button_Interrupt (Num : Button_Num) is
   begin
      IO_Ports.Enable_Interrupt (Butts (Num).Port, Butts (Num).Pin);
   end Enable_Button_Interrupt;
end XSTK_Oled1_Board;
