----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with SAM4S; use SAM4S;
with XSTK_Oled1_Board;
with Timer;
with System.Machine_Code; use System.Machine_Code;
with IO_Ports;
with Ada.Text_IO; use Ada.Text_IO;

package body Butt_Intr is

   Next_Tick : Timer.Tick_Count := 0;

   Init_Butt : array (1 .. 2) of Boolean := (others => False);

   procedure Init
   is
      use XSTK_Oled1_Board;
   begin
      -- set interrupts
      for J in 1 .. 2 loop
         IO_Ports.Enable_Interrupt (Button_Port (J), Button_Pin (J));
      end loop;

      for J in 1 .. 3 loop
         Led_Off (J);
      end loop;
   end Init;

   procedure Button_Intr_Handler (IRQ : IRQ_Id)
   is
      use XSTK_Oled1_Board;
      Butt : Positive;
   begin
      if IRQ = PIOA_IRQ_Id then
         Butt := 1;
      elsif IRQ = PIOC_IRQ_Id then
         Butt := 2;
      else
         raise Program_Error with "Button_Intr_Handler error";
      end if;

      --  check pin number
      --  need read ISR for reset request

      if IO_Ports.Check_Interrupt
        (Button_Port (Butt), Button_Pin (Butt))
      then
         -- первое прерывание происходит по назначению обработчика на порт,
         -- почему? обходим
         if not Init_Butt (Butt) then
            Init_Butt (Butt) := True;
            return;
         end if;

         Led_On (Butt);
         Put_Line (">>> Button Interrupt: " & Integer'Image (Butt));

         --  very long handling interrupt
         Timer.Wait (5000);

         Put_Line ("<<< Button Interrupt: " & Integer'Image (Butt));
         Led_Off (Butt);
      end if;
   end Button_Intr_Handler;


   procedure Timer_Cb (Ticks : Timer.Tick_Count)
   is
      use type Timer.Tick_Count;
   begin
      if Ticks > Next_Tick then
         XSTK_Oled1_Board.Led_Toggle (3);
         Next_Tick := Ticks + 200;
      end if;
   end Timer_Cb;

end Butt_Intr;
