----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with SAM4S; use SAM4S;
with Interfaces; use Interfaces;
with Oled.Draw;
with Oled.Chart;
with IO_Ports;
with Adc;

package body Light_Sens is

   Channel : constant Adc.ADC_Channel_Type := 4;
   Cnt : Integer := 0;

   procedure Init_Display;

   ----------
   -- Init --
   ----------

   procedure Init
   is
      use IO_Ports;
   begin
      Port_Config (Port => Port_B, Pin => 0, Mode => Port_Mode_In);
      Enable_Clock (Adc.ADC_ID);
      Adc.Init (Clock_HZ => 100_000, Startup_Time => 0);
      Adc.Enable_Channel (Channel);
      Adc.Config_Trigger (Trig => Adc.ADC_Trigger.Software,
                          Free_Run => True);
      Adc.Start;

      Init_Display;
   end Init;

   ------------------
   -- Init_Display --
   ------------------

   procedure Init_Display
   is
      use Oled.Chart;
      use Oled.Draw;
   begin
      Oled.Init;
      Draw_String ("Light Sens:", 0, 0);
      Init_Chart (1, 3);
   end Init_Display;

   ------------
   -- Update --
   ------------

   procedure Update
   is
      Y : Word := Adc.Get_Value (Channel);
      YImg : String := Y'Img;
      use Oled.Chart;
      use Oled.Draw;
   begin
      if Unsigned_8 (Cnt) >= Oled.Oled_Col_Num'Last then
         Clear_Chart;
         Cnt := 0;
      else
         Cnt := Cnt + 1;
      end if;
      Draw_Chart (23 - Unsigned_8 (Y / 172), 0);
      Draw_String ("    ", 0, 96);
      Draw_String (YImg (2 .. YImg'Last), 0, 96);
   end Update;
end Light_Sens;
