----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with Timer; use Timer;
with Interfaces; use Interfaces;
with Oled; use Oled;
with Oled.Draw;
with Oled.Chart;
with Oled.Text;
with Oled.Buffered.Float_Oper;
with Bitmaps; use Bitmaps;
with Font;


package body Demo_Proc is

   --  small robot picture
   Simple_Bmp : constant Bitmap :=
        ((2#11110000#,2#00010000#,2#01010000#,2#11010001#,
          2#11110010#,2#01010100#,2#00011000#,2#00010000#,
          2#00011000#,2#01010100#,2#11110010#,2#11010001#,
          2#01010000#,2#00010000#,2#11110000#),
         (2#00111111#,2#00100000#,2#00100000#,2#00100100#,
          2#00100100#,2#01101000#,2#01101010#,2#01101011#,
          2#01101010#,2#01101000#,2#00100100#,2#00100100#,
          2#00100000#,2#00100000#,2#00111111#));

   -- buffers  bitmap 128x32 pix (4 rows, 128 cols)
   Buf_Bmp : Bitmap (0 .. 3, 0 .. 127) := (others => (others => 0));
   -- the bitmap with vertical lines
   Bmp_Ver : Bitmap (0 .. 0, 0 .. 7) := (others => (others => 16#AA#));
   -- the bitmap with horizontal lines
   Bmp_Hor : Bitmap (0 .. 0, 0 .. 7) := (others => (255,0,255,0,255,0,255,0));
   -- step in the demo-procedure
   Demo_Step : Integer := 0;
   Pause : Boolean := False;
   Rnd_Num : Unsigned_8 := 123;

   function Rnd return Unsigned_8;

   ---------------
   -- Demo_Draw --
   ---------------

   procedure Demo_Draw (Start : Boolean := False)  is
      use Oled.Draw;
   begin
      if Start then
         --  draw string in specified position of the screen
         Oled.Start_Line (0);
         Oled.Clear;
         Draw_String ("** Demo Draw **", 0, 0);
         Draw_String ("Run time ", 2, 44);

         Draw_Bitmap (Simple_Bmp,
                      Bmp_Start_Row  => Simple_Bmp'First (1),
                      Bmp_End_Row    => Simple_Bmp'Last (1),
                      Bmp_Start_Col  => Simple_Bmp'First (2),
                      Bmp_End_Col    => Simple_Bmp'Last (2),
                      Oled_Start_Row => 2,
                      Oled_Start_Col => 0);
         Pause := False;
      end if;

      if not Pause then
         Draw_String (Unsigned_32'Image (Timer.Ticks / 1000), 3, 60);
      end if;
   end Demo_Draw;

   ---------------
   -- Test_Text --
   ---------------

   procedure Demo_Text (Start : Boolean := False)
   is
      use Oled.Text;
   begin
      if Start then
         Oled.Start_Line (0);
         Oled.Clear;

         -- use all oled memory (8 pages) for text buffer
         -- 4 visible and 4 invisible (need scroll buffer)
         Set_Text_Buffer (0, 7);
         Put_Line ("** Demo Text **");
         Put_Line ("BUTT1 next demo");
         Put_Line ("BUTT2 pause");
         Put ("abcdefghijkl");
         Put ("mnopqrstuvwxyz");
         New_Line;
         Put_Line ("1234567890;:?<>=");
         Put (" !""#$%&'()*+,-./");

         Demo_Step := 0;
         Pause := False;
      elsif not Pause then
         -- only roll screen buffer
         if Demo_Step < 63 then
            Demo_Step := Demo_Step + 1;
         else
            Demo_Step := 0;
         end if;

          Oled.Start_Line (Unsigned_8 (Demo_Step));
      end if;
   end Demo_Text;

   -----------------
   --  Demo_Chart --
   -----------------

   procedure Demo_Chart_1 (Start : Boolean := False)
   is
      use Oled.Chart;
      use Oled.Draw;
      Y : Unsigned_8;
   begin
      if Start then
         Oled.Start_Line (0);
         Oled.Clear;

         Draw_String ("* Demo Chart 1 *", 0, 0);

         Init_Chart (1, 3);
         Demo_Step := 0;
         Pause := False;
      end if;

      if not Pause then
         if Demo_Step < Integer (Oled.Oled_Col_Num'Last) then
            Demo_Step := Demo_Step + 1;
         else
            Demo_Step := 0;
            for J in 1 .. 3 loop
               Oled.Clear_Page (Unsigned_8 (J));
            end loop;

            Clear_Chart;
         end if;

         Y := Unsigned_8 (Demo_Step) / 16  + Rnd / 32;
         Draw_Chart (Y, Y);
      end if;
   end Demo_Chart_1;

   procedure Demo_Chart_2 (Start : Boolean := False)
   is
      use Oled.Chart;
      use Oled.Draw;
      Y : Unsigned_8;
   begin
      if Start then
         Oled.Start_Line (0);
         Oled.Clear;

         Draw_String ("* Demo Chart 2 *", 0, 0);

         Init_Chart (1, 3);
         Demo_Step := 0;
         Pause := False;
      end if;

      if not Pause then
         if Demo_Step < Integer (Oled.Oled_Col_Num'Last) then
            Demo_Step := Demo_Step + 1;
         else
            Demo_Step := 0;

            for J in 1 .. 3 loop
               Oled.Clear_Page (Unsigned_8 (J));
            end loop;

            Clear_Chart;
         end if;

         Y := Unsigned_8 (Demo_Step) / 16  + Rnd / 24;

         Draw_Chart (Y, 0);
      end if;
   end Demo_Chart_2;

   procedure Demo_Chart_3 (Start : Boolean := False)
   is
      use Oled.Chart;
      use Oled.Draw;
      Y1, Y2 : Unsigned_8;
      Width   : constant Unsigned_8 := 3;
      Space   : constant Unsigned_8 := 3;
   begin
      if Start then
         Oled.Start_Line (0);
         Oled.Clear;

         Draw_String ("* Demo Chart 3 *", 0, 0);

         Init_Chart (1, 3,
                     Column_Width => Width,
                     Column_Space => Space);
         Demo_Step := 0;
         Pause := False;
      end if;

      if not Pause then
         if Demo_Step < Integer (Oled.Oled_Col_Num'Last
                                   /  (Space +  Width)) - 1
         then
            Demo_Step := Demo_Step + 1;
         else
            Demo_Step := 0;

            for J in 1 .. 3 loop
               Oled.Clear_Page (Unsigned_8 (J));
            end loop;

            Clear_Chart;
         end if;

         Y1 := 8 + Unsigned_8 (Demo_Step) / 32 + Rnd / 32;
         Y2 := 8 + Unsigned_8 (Demo_Step) / 32  - Rnd / 32;

         Draw_Chart (Y1, Y2);
      end if;
   end Demo_Chart_3;

   -------------------
   -- Demo_Buffered --
   -------------------

   procedure Demo_Buffered (Start : Boolean := False)
   is
      use Oled.Buffered;
      use Oled.Draw;

      procedure Bmp_Copy (Start_X : Unsigned_8) is
      begin
         for J in Unsigned_8'(0) .. Unsigned_8'(1) loop
            Bitmap_Copy  (Dst_Bmp => Buf_Bmp,
                          Src_Bmp  => Simple_Bmp,
                          Dst_X => Start_X + J * Bitmap_Width (Simple_Bmp),
                          Dst_Y => 0 + J * 3,
                          Src_X1 => 0,
                          Src_X2 => Bitmap_Width (Simple_Bmp) - 1,
                          Src_Y1 => 0,
                          Src_Y2 => Bitmap_Height (Simple_Bmp) - 1);
         end loop;

         Draw_Rect (Buf_Bmp,
                    X1   => Start_X + 7,
                    X2   => Start_X + 19,
                    Y1   => 20,
                    Y2   => 28,
                    Oper => Bit_Copy);

         -- partialy copy of bitmap with XOR operation
         Bitmap_Copy  (Dst_Bmp => Buf_Bmp,
                       Src_Bmp  => Simple_Bmp,
                       Dst_X => Start_X + 7,
                       Dst_Y => 21,
                       Src_X1 => 1,
                       Src_X2 => 13,
                       Src_Y1 => 5,
                       Src_Y2 => 11,
                       Oper => Bit_Xor);
      end Bmp_Copy;

      procedure Draw_Chars (Start_X: Unsigned_8)
      is
         X : Unsigned_8 := Start_X;
         Y : Unsigned_8 := 0;
         Str : String := "TEST";
      begin
         -- draw characters
         for J in 1 .. 4 loop
            if J = 1 then
               X := X + 3;
               Y := Y + 0;
            elsif J = 2 then
               X := X + 5;
               Y := Y + 3;
            elsif J = 3 then
               X := X - 8;
               Y := Y + 5;
            elsif J = 4 then
               X := X + 8;
               Y := Y + 4;
            end if;

            Bitmap_Copy  (Dst_Bmp => Buf_Bmp,
                          Src_Bmp  => Font.Get_Glyph (Str (J)),
                          Dst_X => X,
                          Dst_Y => Y,
                          Oper => Bit_Or);
         end loop;
      end Draw_Chars;

      procedure Draw_Lines (Start_X, Start_Y, End_X, End_Y : Unsigned_8)
      is

         X : Unsigned_8 := Start_X;
         Y : Unsigned_8 := Start_Y;
         Dx : Unsigned_8 := (End_X - Start_X + 1) / 2;
         Dy : constant Unsigned_8 := (End_Y - Start_Y + 1) / 2 ;
      begin
         --  horizontal lines
         X := Start_X;
         Y := Start_Y;

         while Y <= End_Y loop
            Draw_Horiz_Line (Buf_Bmp,
                             Y => Y,
                             X1 => Start_X,
                             X2 => End_X);
            Y := Y + Dy;
         end loop;

         --  vertical lines
         X := Start_X;
         Y := Start_Y;

         while X <= End_X loop
            Draw_Vertic_Line (Buf_Bmp,
                              X => X,
                              Y1 => Start_Y,
                              Y2 => End_Y);
            X := X + Dx;
         end loop;

         --  draw diagonal lines
         Oled.Buffered.Float_Oper.Draw_Line (Buf_Bmp,
                                             X1 => Start_X,
                                             Y1 => Start_Y,
                                             X2 => End_X,
                                             Y2 => End_Y);

         Oled.Buffered.Float_Oper.Draw_Line (Buf_Bmp,
                                             X1 => End_X,
                                             Y1 => Start_Y,
                                             X2 => Start_X,
                                             Y2 => End_Y);
      end Draw_Lines;

      procedure Draw_Rect_Xor (Start_X : Unsigned_8) is
      begin
         Draw_Rect (Buf_Bmp,
                    X1   => Start_X,
                    X2   => Start_X + 19,
                    Y1   => 0,
                    Y2   => 19,
                    Oper => Bit_Copy);

         Draw_Rect (Buf_Bmp,
                    X1   => Start_X + 10,
                    X2   => Start_X + 25,
                    Y1   => 10,
                    Y2   => 25,
                    Oper => Bit_Xor);
      end Draw_Rect_Xor;

   begin
      if Start then
         Oled.Start_Line (0);
         Oled.Clear;

         Demo_Step := 0;
         Pause := False;
      end if;

      if Pause then
         return;
      end if;

      case Demo_Step is
         when 0 =>null;
            Clear_Bitmap (Buf_Bmp, 0);
         when 1 =>
            Draw_Chars (0);
            -- draw lines
         when 2 =>
            -- draw lines
            Draw_Lines (20, 0, 50, 30);
         when 3 =>
            -- draw rectangles with xor
            Draw_Rect_Xor (58);
         when 4 =>
            -- demo copy bitmaps with bit opearation
            Bmp_Copy (90);

         when others => null;
      end case;

      if Demo_Step < 20 then
         Demo_Step := Demo_Step + 1;
      else
         Demo_Step := 0;
      end if;

      -- output buffer to oled
      Draw.Draw_Bitmap (Buf_Bmp,
                        Bmp_Start_Row  => 0,
                        Bmp_End_Row    => Buf_Bmp'Last (1),
                        Bmp_Start_Col  => 0,
                        Bmp_End_Col    => Buf_Bmp'Last (2),
                        Oled_Start_Row => 0,
                        Oled_Start_Col => 0);
   end Demo_Buffered;

   ------------------
   -- Pause_Toggle --
   ------------------

   procedure Pause_Toggle is
   begin
      Pause := not Pause;
   end Pause_Toggle;

   ---------
   -- Rnd --
   ---------

   function Rnd return Unsigned_8
   is
      A : constant Unsigned_32 := 45;
      C : constant Unsigned_32 := 21;
      M : constant Unsigned_32 := 67;
   begin
      Rnd_Num := Unsigned_8 ((A * Unsigned_32 (Rnd_Num) + C) mod 256);
      return Rnd_Num;
   end Rnd;
end Demo_Proc;
