----------------------------------------------------------------------------
-- Copyright (c) 2015 Turishev Evgeniy <ew_turi@mail.ru>                  --
-- This file is licensed under the GNU General Public License version 3   --
--  or any later version.                                                 --
----------------------------------------------------------------------------

with System;
with System.Machine_Code; use System.Machine_Code;
with Interfaces; use Interfaces;
with SAM4S; use SAM4S;
with XSTK_Board;

package body Interrupts is
   Clock_Frequency : constant := 120_000_000;
   Tick_Period   : constant Unsigned_32 := Clock_Frequency / 1000;
   -- period = 1 ms

   System_Vectors : constant System.Address;
   pragma Import (Asm, System_Vectors, "__vectors0");

   Cnt : Integer := 0;

   NL : constant String := ASCII.LF & ASCII.HT;

   procedure Timer_Intr_Handler;
   pragma Export (Asm, Timer_Intr_Handler, "SysTick_Handler");

   ----------
   -- Init --
   ----------

   procedure Init is
   begin
      Asm ("cpsid i", Volatile => True);
      Asm ("cpsid f", Volatile => True);

      --  Initialize vector table
      SYS_CTRL.VTOR := System_Vectors'Address;

      -- init SysTick timer
      SYST.CSR := SYST_CSR.Clk_Source; -- disable clock
      SYST.RVR := Word (Tick_Period - 1);
      SYST.CVR := 0;
      SYST.CSR := SYST_CSR.Clk_Source
        or SYST_CSR.Enable or SYST_CSR.Tick_Int;

      --  enable bus and memory fault traps
      --  SYS_CTRL.SHCSR := SHCSR or 16#7_0000#;

      --  unmask fault
      Asm ("cpsie f", Volatile => True);
      --  unmask interrupts
      --  Asm ("cpsie i", Volatile => True);

      --  init boards IO-ports
      XSTK_Board.Init;
   end Init;

   ------------------------
   -- Timer_Intr_Handler --
   ------------------------

   procedure Timer_Intr_Handler is
   begin
      if Cnt >= 200 then
         Cnt := 0;
         XSTK_Board.LED0_Toggle;
      else
         Cnt := Cnt + 1;
      end if;
   end Timer_Intr_Handler;


   -----------------------------
   -- Interrupt handler stubs --
   -----------------------------

   procedure SUPC_IrqHandler;
   pragma Export (Asm, SUPC_IrqHandler, "SUPC_IrqHandler");
   procedure RSTC_IrqHandler;
   pragma Export (Asm, RSTC_IrqHandler, "RSTC_IrqHandler");
   procedure RTC_IrqHandler;
   pragma Export (Asm, RTC_IrqHandler, "RTC_IrqHandler");
   procedure RTT_IrqHandler;
   pragma Export (Asm, RTT_IrqHandler, "RTT_IrqHandler");
   procedure WDT_IrqHandler;
   pragma Export (Asm, WDT_IrqHandler, "WDT_IrqHandler");
   procedure PMC_IrqHandler;
   pragma Export (Asm, PMC_IrqHandler, "PMC_IrqHandler");
   procedure EEFC_IrqHandler;
   pragma Export (Asm, EEFC_IrqHandler, "EEFC_IrqHandler");
   procedure IrqHandlerNotUsed;
   pragma Export (Asm, IrqHandlerNotUsed, "IrqHandlerNotUsed");
   procedure UART0_IrqHandler;
   pragma Export (Asm, UART0_IrqHandler, "UART0_IrqHandler");
   procedure UART1_IrqHandler;
   pragma Export (Asm, UART1_IrqHandler, "UART1_IrqHandler");
   procedure SMC_IrqHandler;
   pragma Export (Asm, SMC_IrqHandler, "SMC_IrqHandler");
   procedure PIOA_IrqHandler;
   pragma Export (Asm, PIOA_IrqHandler, "PIOA_IrqHandler");
   procedure PIOB_IrqHandler;
   pragma Export (Asm, PIOB_IrqHandler, "PIOB_IrqHandler");
   procedure PIOC_IrqHandler;
   pragma Export (Asm, PIOC_IrqHandler, "PIOC_IrqHandler");
   procedure USART0_IrqHandler;
   pragma Export (Asm, USART0_IrqHandler, "USART0_IrqHandler");
   procedure USART1_IrqHandler;
   pragma Export (Asm, USART1_IrqHandler, "USART1_IrqHandler");
   procedure USART2_IrqHandler;
   pragma Export (Asm, USART2_IrqHandler, "USART2_IrqHandler");
   procedure MCI_IrqHandler;
   pragma Export (Asm, MCI_IrqHandler, "MCI_IrqHandler");
   procedure TWI0_IrqHandler;
   pragma Export (Asm, TWI0_IrqHandler, "TWI0_IrqHandler");
   procedure TWI1_IrqHandler;
   pragma Export (Asm, TWI1_IrqHandler, "TWI1_IrqHandler");
   procedure SPI_IrqHandler;
   pragma Export (Asm, SPI_IrqHandler, "SPI_IrqHandler");
   procedure SSC_IrqHandler;
   pragma Export (Asm, SSC_IrqHandler, "SSC_IrqHandler");
   procedure TC0_IrqHandler;
   pragma Export (Asm, TC0_IrqHandler, "TC0_IrqHandler");
   procedure TC1_IrqHandler;
   pragma Export (Asm, TC1_IrqHandler, "TC1_IrqHandler");
   procedure TC2_IrqHandler;
   pragma Export (Asm, TC2_IrqHandler, "TC2_IrqHandler");
   procedure TC3_IrqHandler;
   pragma Export (Asm, TC3_IrqHandler, "TC3_IrqHandler");
   procedure TC4_IrqHandler;
   pragma Export (Asm, TC4_IrqHandler, "TC4_IrqHandler");
   procedure TC5_IrqHandler;
   pragma Export (Asm, TC5_IrqHandler, "TC5_IrqHandler");
   procedure ADC_IrqHandler;
   pragma Export (Asm, ADC_IrqHandler, "ADC_IrqHandler");
   procedure DAC_IrqHandler;
   pragma Export (Asm, DAC_IrqHandler, "DAC_IrqHandler");
   procedure PWM_IrqHandler;
   pragma Export (Asm, PWM_IrqHandler, "PWM_IrqHandler");
   procedure CRCCU_IrqHandler;
   pragma Export (Asm, CRCCU_IrqHandler, "CRCCU_IrqHandler");
   procedure ACC_IrqHandler;
   pragma Export (Asm, ACC_IrqHandler, "ACC_IrqHandler");
   procedure USBD_IrqHandler;
   pragma Export (Asm, USBD_IrqHandler, "USBD_IrqHandler");

   procedure SUPC_IrqHandler is
   begin
      null;
   end SUPC_IrqHandler;

   procedure RSTC_IrqHandler is
   begin
      null;
   end RSTC_IrqHandler;

   procedure RTC_IrqHandler is
   begin
      null;
   end RTC_IrqHandler;

   procedure RTT_IrqHandler is
   begin
      null;
   end RTT_IrqHandler;

   procedure WDT_IrqHandler is
   begin
      null;
   end WDT_IrqHandler;

   procedure PMC_IrqHandler is
   begin
      null;
   end PMC_IrqHandler;

   procedure EEFC_IrqHandler is
   begin
      null;
   end EEFC_IrqHandler;

   procedure IrqHandlerNotUsed is
   begin
      null;
   end IrqHandlerNotUsed;

   procedure UART0_IrqHandler is
   begin
      null;
   end UART0_IrqHandler;

   procedure UART1_IrqHandler is
   begin
      null;
   end UART1_IrqHandler;

   procedure SMC_IrqHandler is
   begin
      null;
   end SMC_IrqHandler;

   procedure PIOA_IrqHandler is
   begin
      null;
   end PIOA_IrqHandler;

   procedure PIOB_IrqHandler is
   begin
      null;
   end PIOB_IrqHandler;

   procedure PIOC_IrqHandler is
   begin
      null;
   end PIOC_IrqHandler;

   procedure USART0_IrqHandler is
   begin
      null;
   end USART0_IrqHandler;

   procedure USART1_IrqHandler is
   begin
      null;
   end USART1_IrqHandler;

   procedure USART2_IrqHandler is
   begin
      null;
   end USART2_IrqHandler;

   procedure MCI_IrqHandler is
   begin
      null;
   end MCI_IrqHandler;

   procedure TWI0_IrqHandler is
   begin
      null;
   end TWI0_IrqHandler;

   procedure TWI1_IrqHandler is
   begin
      null;
   end TWI1_IrqHandler;

   procedure SPI_IrqHandler is
   begin
      null;
   end SPI_IrqHandler;

   procedure SSC_IrqHandler is
   begin
      null;
   end SSC_IrqHandler;

   procedure TC0_IrqHandler is
   begin
      null;
   end TC0_IrqHandler;

   procedure TC1_IrqHandler is
   begin
      null;
   end TC1_IrqHandler;

   procedure TC2_IrqHandler is
   begin
      null;
   end TC2_IrqHandler;

   procedure TC3_IrqHandler is
   begin
      null;
   end TC3_IrqHandler;

   procedure TC4_IrqHandler is
   begin
      null;
   end TC4_IrqHandler;

   procedure TC5_IrqHandler is
   begin
      null;
   end TC5_IrqHandler;

   procedure ADC_IrqHandler is
   begin
      null;
   end ADC_IrqHandler;

   procedure DAC_IrqHandler is
   begin
      null;
   end DAC_IrqHandler;

   procedure PWM_IrqHandler is
   begin
      null;
   end PWM_IrqHandler;

   procedure CRCCU_IrqHandler is
   begin
      null;
   end CRCCU_IrqHandler;

   procedure ACC_IrqHandler is
   begin
      null;
   end ACC_IrqHandler;

   procedure USBD_IrqHandler is
   begin
      null;
   end USBD_IrqHandler;

begin
   Init;
end Interrupts;
